<?php
if ( ! defined( 'ABSPATH' ) ) exit;

  class Fasta_Credit_Gateway extends WC_Payment_Gateway {

    function __construct() {
      $this->id = 'fasta-credit';
      $this->method_title = __( 'FASTA Credit for Checkout', 'fasta-credit' );
      $this->method_description = __( 'Allow your customers to get instant credit on your WooCommerce Store', 'fasta-credit' );
      $this->title = __( 'Instant Credit', 'fasta-credit' );
      $this->has_fields = true;
      $this->supports = array( 'products' );
      $this->logger = Fasta_Credit_Utils::is_wc_3() ? wc_get_logger() : new WC_Logger();
      $this->logger_context = array( 'source' => $this->id );
      $this->init_form_fields();
      $this->init_settings();
      $this->init_plugin_settings();
      $this->icon = plugin_dir_url( __DIR__ ) . 'assets/fasta-logo-woocommerce.png';
      $this->in_progress_statuses = array( 'on-hold', 'pending' );

      if ( is_admin() ) {
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
      }

      $session = WC()->session;
      if ( isset( $session ) ) {
        $chosen_payment_method = $session->get('chosen_payment_method');
        if ( $chosen_payment_method == "fasta-credit" ) {
          add_action( 'wp_enqueue_scripts', function() {
            Fasta_Credit_Utils::load_fasta_checkout_scripts( $this->fasta_url, $this->merchant_id );
          });
        }
      }
      add_action( 'woocommerce_api_'.strtolower( get_class( $this ) ), array( $this, 'handle_callback' ) );
    }

    function validate_merchant_id_dev_field( $key, $value ) {
      if ( isset( $value ) && $value == '' ) {
        WC_Admin_Settings::add_error( esc_html__( 'Your sandbox merchant ID must be added.', 'fasta-credit' ) );
      }
      return $value;
    }

    function validate_merchant_id_prod_field( $key, $value ) {
      if ( isset( $value ) && $value == '' && $this->settings['dev_mode_enabled'] == 'no' ) {
        WC_Admin_Settings::add_error( esc_html__( 'Your production merchant ID must be added.', 'fasta-credit' ) );
      }
      return $value;
    }

    function init_form_fields() {
      $log_label = '';
      if ( defined( 'WC_LOG_DIR' ) ) {
        $log_url = add_query_arg( 'tab', 'logs', add_query_arg( 'page', 'wc-status', admin_url( 'admin.php' ) ) );
        $log_key = $this->id . sanitize_file_name( wp_hash( $this->id ) ) . '-log';
        $log_url = add_query_arg( 'log_file', $log_key, $log_url );
      
        $log_label .= ' | ' . sprintf( __( '%1$sView Log%2$s', 'your-textdomain-here' ), '<a href="' . esc_url( $log_url ) . '">', '</a>' );
      }

      $this->form_fields = array(
        'enabled' => array(
          'title'   => __( 'Enabled', 'fasta-credit' ),
          'label'   => __( 'Enable <i>FASTA Credit for Checkout</i>', 'fasta-credit' ),
          'type'    => 'checkbox',
          'default' => 'no'
        ),
        'fasta_embedded' => array(
          'title'   => __( 'FASTA Embedded', 'fasta-credit' ),
          'label' => 'Use the embedded version of FASTA',
          'description' => __( 'If enabled FASTA will be embedded inside your website so that your customers never leave your store. If you want your customers to be redirected to FASTA instead uncheck this option.', 'fasta-credit' ),
          'type'    => 'checkbox',
          'default' => 'yes'
        ),
        'dev_mode_enabled' => array(
          'title'   => __( 'Sandbox Mode', 'fasta-credit' ),
          'label'   => __( 'Use the FASTA sandbox environment', 'fasta-credit' ),
          'description' => __('<b>Note:</b> Un-checking this option will use the FASTA <b>production</b> environment.', 'fasta-credit'),
          'type'    => 'checkbox',
          'default' => 'yes'
        ),
        'merchant_id_dev' => array(
          'title'    => __( 'Sandbox Merchant ID', 'fasta-credit' ),
          'type'     => 'text',
          'desc_tip' => __( 'Your unique sandbox merchant ID you received after registering with FASTA.', 'fasta-credit' ),
          'default' => ''
        ),
        'merchant_api_key_dev' => array(
          'title'    => __( 'Sandbox API Key', 'fasta-credit' ),
          'type'     => 'text',
          'desc_tip' => __( 'Your sandbox API used to verify the loan status once completed.', 'fasta-credit' ),
          'default' => ''
        ),
        'merchant_id_prod' => array(
          'title'    => __( 'Production Merchant ID', 'fasta-credit' ),
          'type'     => 'text',
          'desc_tip' => __( 'Your unique production merchant ID you received after registering with FASTA.', 'fasta-credit' ),
          'default' => ''
        ),
        'merchant_api_key_prod' => array(
          'title'    => __( 'Production API Key', 'fasta-credit' ),
          'type'     => 'text',
          'desc_tip' => __( 'Your production API used to verify the loan status once completed.', 'fasta-credit' ),
          'default' => ''
        ),
        'wc_fasta_credit_debug' => array(
          'title'    => __( 'Enable Debug Logs', 'fasta-credit' ),
          'label' => $log_label,
          'type'     => 'checkbox',
          'desc_tip' => __( 'Allow FASTA to write debug logs.', 'fasta-credit' ),
          'default' => 'no'
        )
      );
    }

    function payment_fields() {
      if ( $this->enabled == 'no' ) {
        echo 'FASTA has not been enabled by the store owner yet. Please contact the owner or select a different payment method.';
        return;
      }

      echo 
        '<span>
          FASTA gives you access to instant credit when you checkout, and in minutes.
          When your application is approved, FASTA pays the retailer directly and your purchase is sent to you.
        </span>';
    }

    function process_payment( $order_id ) {
      $order = new WC_Order( $order_id );
      $redirect_url = $this->fasta_embedded == 'yes' ? $order->get_checkout_payment_url( $on_checkout = true ) : $this->fasta_url . '/?platform=checkout&inline=true&checkoutInfo=' . json_encode( Fasta_Credit_Utils::build_checkout_info( $order, $this->merchant_id ) );

      return array(
        'result'   => 'success',
        'redirect' => $redirect_url
      );
    }

    function handle_callback() {
      try {
        $entity_body = file_get_contents( 'php://input' );
        $json_body = json_decode( $entity_body, true );
        
        $order_id = $json_body['checkout']['cartId'];
        $order = new WC_Order( $order_id );
        $order_status = strtolower( $order->get_status() );
        
        if ( !in_array( $order_status,  $this->in_progress_statuses ) ) {
          throw new Exception( "Invalid attempt to change {$order_status} order #{$order_id}." );
        }
        
        $loan_status = $json_body['status'];
        $loan_id = $json_body['loanId'];
        $loan_number = $json_body['loanNumber'] ?? '';
        $received_checkout_amount = (float) $json_body['checkout']['amount'];
        $responseMessage = '';
        $merchant_portal_link = "<a href='". Fasta_Credit_Utils::build_fasta_merchant_portal_url( $this->fasta_merchant_portal_url )( $loan_number ) ."' target='_blank'>application {$loan_number}</a>";
        
        $payment_result = $this->get_payment_result( $loan_id );
        $confirmed_checkout_amount = (float) $payment_result['checkoutInfo']['amount'];
        
        if( $confirmed_checkout_amount != $received_checkout_amount ) {
          $order->add_order_note( __( "The received FASTA cart amount did not match confirmed cart amount. Visit {$merchant_portal_link} for more detail.", 'fasta-credit' ) );
        } else if ( $loan_status === TRUE ) {
          $order->add_order_note( __( "FASTA {$merchant_portal_link} was successful.", 'fasta-credit' ) );
          $order->payment_complete();
          $responseMessage = "Updated status of order #{$order_id} to {$order->get_status()}.";
        } else if ( $loan_status === FALSE ) {
          $order->add_order_note( __( "FASTA {$merchant_portal_link} was unsuccessful. Order is {$order_status}.", 'fasta-credit' ) );
          $responseMessage = "Loan unsuccessful. Order #{$order_id} status remains {$order_status}.";
        } else {
          $order->add_order_note( __( "FASTA {$merchant_portal_link} is in progress. Order is {$order_status}.", 'fasta-credit' ) );
          $responseMessage = "Loan in progress. Order #{$order_id} status remains {$order_status}.";
        }
        
        wp_send_json_success( array(
          'message' => $responseMessage
        ) );
        exit;
      } catch (Exception $e) {
        $message = $e->getMessage();
        $this->fasta_logger( 'debug', $message . "\n" . $e->getTraceAsString() );

        wp_send_json_error( array(
          'error' => $message
        ), 500 );
        exit;
      }
    }

    function init_plugin_settings() {
      foreach ( $this->settings as $setting_key => $value ) {
        $this->$setting_key = $value;
      }
      $this->merchant_id = $this->dev_mode_enabled == 'yes' ? $this->merchant_id_dev : $this->merchant_id_prod;
      $this->fasta_url = $this->dev_mode_enabled == 'yes' ? 'https://dev.fasta.co.za' : 'https://www.fasta.co.za';
      $this->fasta_api_url = $this->dev_mode_enabled == 'yes' ? 'https://api.dev.fasta.co.za' : 'https://api.fasta.co.za';
      $this->api_key = $this->dev_mode_enabled == 'yes' ? $this->merchant_api_key_dev : $this->merchant_api_key_prod;
      $this->fasta_merchant_portal_url = $this->dev_mode_enabled == 'yes' ? 'https://admin.dev.fasta.co.za' : 'https://admin.fasta.co.za';
    }

    function fasta_logger( string $level, string $message ) {
      if( $this->wc_fasta_credit_debug == 'yes' ) {
        Fasta_Credit_Utils::is_wc_3() ? $this->logger->log( $level, $message, $this->logger_context ) : $this->logger->add( $this->id, $message );
      }
    }

    function get_payment_result( string $loan_id) {
      try {
        $args = array(
          'headers' => array(
            'Content-Type' => 'application/json',
            'Authorization' => 'Basic ' . base64_encode( "{$this->merchant_id}:{$this->api_key}" )
          )
        );

        $url = $this->fasta_api_url . "/payment-result/merchant/loan/?loanId={$loan_id}";
        $response = wp_remote_get( $url, $args );

        if ( is_wp_error( $response ) ) {
          throw new Exception( $response->get_error_message() );
        }
        return json_decode( $response['body'], true );
      } catch (Exception $ex) {
        throw $ex;
      }
    }
  }
?>